Skip to contents

This is the dataset used for case study 3.

Format

A data frame with 1110 rows and 12 variables

Details

This case study examines reproductive patterns over multiple temporal scales in two tropical intertidal gastropods.