This is the dataset used for case study 2.
A data frame with 285 rows and 21 variables
Details
This case study reanalyses data from Langlois et al. (2005) to
examine whether large reef-associated predators (rock lobster and snapper)
structure adjacent soft‑sediment communities.