R/data.R
case_study3.Rd
This is the dataset used for case study 3.
A data frame with 1110 rows and 12 variables
This case study examines reproductive patterns over multiple temporal scales in two tropical intertidal gastropods.